@@ -1,30 +1,6 @@
-/*
-Copyright (c) 2009-2010 Sony Pictures Imageworks Inc., et al.
-All Rights Reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are
-met:
-* Redistributions of source code must retain the above copyright
-  notice, this list of conditions and the following disclaimer.
-* Redistributions in binary form must reproduce the above copyright
-  notice, this list of conditions and the following disclaimer in the
-  documentation and/or other materials provided with the distribution.
-* Neither the name of Sony Pictures Imageworks nor the names of its
-  contributors may be used to endorse or promote products derived from
-  this software without specific prior written permission.
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright Contributors to the Open Shading Language project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/imageworks/OpenShadingLanguage
 
 #pragma once
 
@@ -36,7 +12,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <OpenImageIO/refcnt.h>
 #include <OpenImageIO/ustring.h>
-#include <OpenImageIO/array_view.h>
+
 
 OSL_NAMESPACE_ENTER
 
@@ -52,18 +28,30 @@ class ShaderSymbol;
 
 /// Opaque pointer to whatever the renderer uses to represent a
 /// (potentially motion-blurred) coordinate transformation.
-typedef const void * TransformationPtr;
+typedef const void* TransformationPtr;
 
 
 // Callbacks for closure creation
-typedef void (*PrepareClosureFunc)(RendererServices *, int id, void *data);
-typedef void (*SetupClosureFunc)(RendererServices *, int id, void *data);
+typedef void (*PrepareClosureFunc)(RendererServices*, int id, void* data);
+typedef void (*SetupClosureFunc)(RendererServices*, int id, void* data);
 
 
 namespace pvt {
-    class ShadingSystemImpl;
+class ShadingSystemImpl;
 }
 
+namespace Strings {
+#ifdef __CUDA_ARCH__
+    #define STRDECL(str,var_name) extern __device__ ustring var_name;
+#else
+    // Any strings referenced inside of a libsoslexec/wide/*.cpp
+    // or liboslnoise/wide/*.cpp will need OSLEXECPUBLIC
+    #define STRDECL(str,var_name) OSLEXECPUBLIC extern const ustring var_name;
+#endif
+#include <OSL/strdecls.h>
+#undef STRDECL
+}; // namespace Strings
+
 
 
 class OSLEXECPUBLIC ShadingSystem
@@ -80,6 +68,7 @@ public:
     /// 1. Attributes that should be exposed to users:
     ///    int statistics:level   Automatically print OSL statistics (0).
     ///    string searchpath:shader  Colon-separated path to search for .oso
+    ///                                files ("", meaning test "." only)
     ///    string colorspace      Name of RGB color space ("Rec709")
     ///    int range_checking     Generate extra code for component & array
     ///                              range checking (1)
@@ -98,6 +87,10 @@ public:
     ///    int no_noise           Replace noise with constant value. (0)
     ///    int no_pointcloud      Skip pointcloud lookups. (0)
     ///    int exec_repeat        How many times to run each group (1).
+    ///    int opt_warnings       Warn on certain failure to runtime-optimize
+    ///                              certain shader constructs. (0)
+    ///    int gpu_opt_error      Consider a hard error if certain shader
+    ///                              constructs cannot be optimized away. (0)
     /// 2. Attributes that should be set by applications/renderers that
     /// incorporate OSL:
     ///    string commonspace     Name of "common" coord system ("world")
@@ -111,23 +104,48 @@ public:
     ///                              fails to find the layer or parameter? (1)
     ///    int strict_messages    Issue error if a message is set after
     ///                              being queried (1).
+    ///    int error_repeats      If zero, suppress repeats of errors and
+    ///                              warnings that are exact duplicates of
+    ///                              earlier ones. (1)
     ///    int lazylayers         Evaluate shader layers only when their
     ///                              outputs are first needed (1)
     ///    int lazyglobals        Run layers lazily even if they write to
     ///                              globals (1)
     ///    int lazyunconnected    Run layers lazily even if they have no
     ///                              output connections (1). For debugging.
+    ///    int lazyerror          Run layers lazily even if they have error
+    ///                              ops after optimization (1).
     ///    int lazy_userdata      Retrieve userdata lazily (0).
     ///    int userdata_isconnected  Should lockgeom=0 params (that may
     ///                              receive userdata) return true from
     ///                              isconnected()? (0)
     ///    int greedyjit          Optimize and compile all shaders up front,
     ///                              versus only as needed (0).
+    ///    int llvm_target_host   Target the specific host architecture for
+    ///                              LLVM IR generation. (1)
+    ///    int llvm_jit_fma       Allow fused mul/add (0). This can increase
+    ///                              speed but can change rounding accuracy
+    ///                              (generally by being better), and might
+    ///                              differ by hardware platform.
+    ///    string llvm_jit_target  JIT to a specific ISA: "" or "none" means
+    ///                              no special ops, "x64", "SSE4.2", "AVX",
+    ///                              "AVX2", "AVX2_noFMA", "AVX512",
+    ///                              "AVX512_noFMA", or "host" means to
+    ///                              figure out what the host can do. ("")
+    ///    int llvm_jit_aggressive  Use LLVM "aggressive" JIT mode. (0)
+    ///    int vector_width       Vector width to allow for SIMD ops (4).
+    ///    int llvm_debugging_symbols  When JITing, generate debug symbols
+    ///                             that associate machine code with shader
+    ///                             source and lines. (0)
+    ///    int llvm_profiling_events  When JITing, generate events to enable
+    ///                             full profiling of shaders. (0)
     ///    int lockgeom           Default 'lockgeom' value for shader params
     ///                              that don't specify it (1).  Lockgeom
     ///                              means a param CANNOT be overridden by
     ///                              interpolated geometric parameters.
     ///    int countlayerexecs    Add extra code to count total layers run.
+    ///    int allow_shader_replacement Allow shader to be specified more than
+    ///                              once, replacing former definition.
     ///    string archive_groupname  Name of a group to pickle and archive.
     ///    string archive_filename   Name of file to save the group archive.
     /// 3. Attributes that that are intended for developers debugging
@@ -152,6 +170,12 @@ public:
     ///                              layer functions.
     ///    int llvm_debug_ops     Extra printfs for each OSL op (helpful
     ///                              for devs to find crashes)
+    ///    int llvm_output_bitcode  Output the full bitcode for each group,
+    ///                              for debugging. (0)
+    ///    int llvm_dumpasm       Print the CPU assembly code from the JIT (0)
+    ///    string llvm_prune_ir_strategy  Strategy for pruning unnecessary
+    ///                              IR (choices: "prune" [default],
+    ///                              "internalize", or "none").
     ///    int max_local_mem_KB   Error if shader group needs more than this
     ///                              much local storage to execute (1024K)
     ///    string debug_groupname Name of shader group -- debug only this one
@@ -267,6 +291,12 @@ public:
     ///   int unknown_closures_needed  Nonzero if additional closures may be
     ///                                needed, whose names can't be known
     ///                                without actually running the shader.
+    ///   int globals_read           Bitfield ("or'ed" SGBits values) of
+    ///                                which ShaderGlobals may be read by
+    ///                                by the shader group.
+    ///   int globals_write         Bitfield ("or'ed" SGBits values) of
+    ///                                which ShaderGlobals may be written by
+    ///                                by the shader group.
     ///   int num_globals_needed     The number of named globals needed.
     ///   ptr globals_needed         Retrieves a pointer to the ustring array
     ///                                containing all globals needed.
@@ -292,7 +322,7 @@ public:
     ///   ptr attribute_scopes       Retrieves a pointer to a ustring array containing
     ///                                the scopes associated with each attribute query
     ///                                in the attributes_needed array.
-    ///   int unknown_attributes_needed  Nonzero if additonal attributes may be
+    ///   int unknown_attributes_needed  Nonzero if additional attributes may be
     ///                                  needed, whose names will not be known
     ///                                  until the shader actually runs.
     ///   int num_renderer_outputs   Number of named renderer outputs.
@@ -347,17 +377,21 @@ public:
     //    /* First layer - texture lookup shader: */
     //       /* Specify instance parameter values */
     //       const char *mapname = "colormap.exr";
-    //       ss->Parameter ("texturename", TypeDesc::TypeString, &mapname);
+    //       ss->Parameter (*group, "texturename", mapname);
     //       float blur = 0.001;
-    //       ss->Parameter ("blur", TypeDesc::TypeFloat, &blur);
+    //       ss->Parameter (*group, "blur", blur);
+    //       Vec3 colorfilter (0.5f, 0.5f, 1.0f);
+    //       ss->Parameter (*group, "colorfilter", TypeDesc::TypeColor,
+    //                      &colorfilter);
     //    ss->Shader ("surface", "texmap", "texturelayer");
     //    /* Second layer - generate the BSDF closure: */
     //       float roughness = 0.05;
-    //       ss->Parameter ("roughness", TypeDesc::TypeFloat, &roughness);
-    //    ss->Shader ("surface", "plastic", "illumlayer");
+    //       ss->Parameter (*group, "roughness", roughness);
+    //    ss->Shader (*group, "surface", "plastic", "illumlayer");
     //    /* Make a connection between the layers */
-    //    ss->ConnectShaders ("texturelayer", "Cout", "illumlayer", "Cs");
-    // ss->ShaderGroupEnd ();
+    //    ss->ConnectShaders (*group, "texturelayer", "Cout",
+    //                       "illumlayer", "Cs");
+    // ss->ShaderGroupEnd (*group);
 
     /// Signal the start of a new shader group.  The return value is a
     /// reference-counted opaque handle to the ShaderGroup.
@@ -377,64 +411,125 @@ public:
 
     /// Signal the end of a new shader group.
     ///
-    bool ShaderGroupEnd (void);
+    bool ShaderGroupEnd (ShaderGroup& group);
 
-    /// Set a parameter of the next shader.
+    /// Set a parameter of the next shader that will be added to the group,
+    /// optionally setting the 'lockgeom' metadata for that parameter
+    /// (despite how it may have been set in the shader).  If lockgeom is
+    /// false, it means that this parameter should NOT be considered locked
+    /// against changes by the geometry, and therefore the shader should not
+    /// optimize assuming that the instance value (the 'val' specified by
+    /// this call) is a constant.
+    bool Parameter (ShaderGroup& group, string_view name, TypeDesc t,
+                    const void *val, bool lockgeom=true);
+    // Shortcuts for param passing a single int, float, or string.
+    bool Parameter (ShaderGroup& group, string_view name,
+                    int val, bool lockgeom=true) {
+        return Parameter (group, name, TypeDesc::INT, &val, lockgeom);
+    }
+    bool Parameter (ShaderGroup& group, string_view name,
+                    float val, bool lockgeom=true) {
+        return Parameter (group, name, TypeDesc::FLOAT, &val, lockgeom);
+    }
+    bool Parameter (ShaderGroup& group, string_view name,
+                    const std::string& val, bool lockgeom=true) {
+        const char *s = val.c_str();
+        return Parameter (group, name, TypeDesc::STRING, &s, lockgeom);
+    }
+    bool Parameter (ShaderGroup& group, string_view name,
+                    ustring val, bool lockgeom=true) {
+        return Parameter (group, name, TypeDesc::STRING, (const char**)&val, lockgeom);
+    }
+
+    /// Append a new shader instance onto the specified group. The shader
+    /// instance will get any pending parameters that were set by
+    /// Parameter() calls since the last Shader() call for the group.
+    bool Shader (ShaderGroup& group, string_view shaderusage,
+                 string_view shadername, string_view layername);
+
+    /// Connect two shaders within the specified group. The source layer
+    /// must be *upstream* of down destination layer (i.e. source must be
+    /// declared earlier within the shader group). The named parameters must
+    /// be of compatible type -- float to float, color to color, array to
+    /// array of the same length and element type, etc. In general, it is
+    /// permissible to connect type A to type B if and only if it is allowed
+    /// within OSL to assign an A to a B (i.e., if `A = B` is legal). So any
+    /// "triple" may be connected to any other triple, and a float output
+    /// may be connected to a triple input (but not the other way around).
+    /// It is permitted to connect a single component of an aggregate to a
+    /// float and vice versa, for example,
+    ///   `ConnectShaders (group, "lay1", "mycolorout[2]",
+    ///                    "lay2", "myfloatinput")`
     ///
-    bool Parameter (string_view name, TypeDesc t, const void *val);
-
-    /// Set a parameter of the next shader, and override the 'lockgeom'
-    /// metadata for that parameter (despite how it may have been set in
-    /// the shader).  If lockgeom is false, it means that this parameter
-    /// should NOT be considered locked against changes by the geometry,
-    /// and therefore the shader should not optimize assuming that the
-    /// instance value (the 'val' specified by this call) is a constant.
-    bool Parameter (string_view name, TypeDesc t, const void *val,
-                    bool lockgeom);
-
-    /// Create a new shader instance, either replacing the one for the
-    /// specified usage (if not within a group) or appending to the
-    /// current group (if a group has been started).
-    bool Shader (string_view shaderusage,
-                 string_view shadername = string_view(),
-                 string_view layername = string_view());
-
-    /// Connect two shaders within the current group
-    ///
-    bool ConnectShaders (string_view srclayer, string_view srcparam,
+    bool ConnectShaders (ShaderGroup &group,
+                         string_view srclayer, string_view srcparam,
                          string_view dstlayer, string_view dstparam);
 
     /// Replace a parameter value in a previously-declared shader group.
     /// This is meant to called after the ShaderGroupBegin/End, but will
     /// fail if the shader has already been irrevocably optimized/compiled,
-    /// unless the paraticular parameter is marked as lockgeom=0 (which
+    /// unless the particular parameter is marked as lockgeom=0 (which
     /// indicates that it's a parameter that may be overridden by the
     /// geometric primitive).  This call gives you a way of changing the
     /// instance value, even if it's not a geometric override.
     bool ReParameter (ShaderGroup &group,
                       string_view layername, string_view paramname,
                       TypeDesc type, const void *val);
+    // Shortcuts for param passing a single int, float, or string.
+    bool ReParameter (ShaderGroup &group, string_view layername,
+                      string_view paramname, int val) {
+        return ReParameter (group, layername, paramname, TypeDesc::INT, &val);
+    }
+    bool ReParameter (ShaderGroup &group, string_view layername,
+                      string_view paramname, float val) {
+        return ReParameter (group, layername, paramname, TypeDesc::FLOAT, &val);
+    }
+    bool ReParameter (ShaderGroup &group, string_view layername,
+                      string_view paramname, const std::string& val) {
+        const char *s = val.c_str();
+        return ReParameter (group, layername, paramname, TypeDesc::STRING, &s);
+    }
+    bool ReParameter (ShaderGroup &group, string_view layername,
+                      string_view paramname, ustring val) {
+        return ReParameter (group, layername, paramname, TypeDesc::STRING,
+                            (const char**)&val);
+    }
 
-    /// Optional: create the per-thread data needed for shader
-    /// execution.  Doing this and passing it to get_context speeds is a
-    /// bit faster than get_context having to do a thread-specific
-    /// lookup on its own, but if you do it, it's important for the app
-    /// to use one and only one PerThreadInfo per renderer thread, and
-    /// destroy it with destroy_thread_info when the thread terminates.
+    // Non-threadsafe versions of Parameter, Shader, ConnectShaders, and
+    // ShaderGroupEnd. These depend on some persistent state about which
+    // shader group is the "current" one being amended. It's fine to use
+    // that as long as all shader specification is done from one thread only
+    // (or at least that you are sure no two groups are being specified
+    // concurrently). If there is any doubt about that, use the versions
+    // above that take an explicit `ShaderGroup&`, which are thread-safe
+    // and re-entrant.
+    bool Parameter (string_view name, TypeDesc t, const void *val,
+                    bool lockgeom=true);
+    bool Shader (string_view shaderusage, string_view shadername,
+                 string_view layername);
+    bool ConnectShaders (string_view srclayer, string_view srcparam,
+                         string_view dstlayer, string_view dstparam);
+    bool ShaderGroupEnd (void);
+
+    /// Create a per-thread data needed for shader execution.  It's very
+    /// important for the app to never use a PerThreadInfo from more than
+    /// one thread (and probably a good idea allocate only one PerThreadInfo
+    /// for each renderer thread), and destroy it with destroy_thread_info
+    /// when the thread terminates (and before the ShadingSystem is
+    /// destroyed).
     PerThreadInfo * create_thread_info();
 
     /// Destroy a PerThreadInfo that was allocated by
     /// create_thread_info().
     void destroy_thread_info (PerThreadInfo *threadinfo);
 
-    /// Get a ShadingContext that we can use.  The context is specific
-    /// to the renderer thread.  The 'threadinfo' parameter should be a
-    /// thread-specific pointer created by create_thread_info, or NULL,
-    /// in which case the ShadingSystem will do the thread-specific
-    /// lookup automatically (and at some additional cost).  The context
+    /// Get a ShadingContext that we can use.  The context is specific to a
+    /// renderer thread, and should never be passed between or shared by
+    /// more than one thread.  The 'threadinfo' parameter should be a
+    /// thread-specific pointer created by create_thread_info.  The context
     /// can be used to shade many points; a typical usage is to allocate
     /// just one context per thread and use it for the whole run.
-    ShadingContext *get_context (PerThreadInfo *threadinfo=NULL,
+    ShadingContext *get_context (PerThreadInfo *threadinfo,
                                  TextureSystem::Perthread *texture_threadinfo=NULL);
 
     /// Return a ShadingContext to the pool.
@@ -447,12 +542,13 @@ public:
     /// execute_init, execute_layer of the last (presumably group entry)
     /// layer, and execute_cleanup. If run==false, just do the binding and
     /// setup, don't actually run the shader.
-    bool execute (ShadingContext *ctx, ShaderGroup &group,
-                  ShaderGlobals &globals, bool run=true);
-    OSL_DEPRECATED("Deprecated since 1.6, pass context pointer, not reference.")
     bool execute (ShadingContext &ctx, ShaderGroup &group,
                   ShaderGlobals &globals, bool run=true);
 
+    // DEPRECATED(2.0): ctx pointer
+    bool execute (ShadingContext *ctx, ShaderGroup &group,
+                  ShaderGlobals &globals, bool run=true);
+
     /// Bind a shader group and globals to the context, in preparation to
     /// execute, including optimization and JIT of the group (if it has not
     /// already been done).  If 'run' is true, also run any initialization
@@ -530,7 +626,7 @@ public:
     TypeDesc symbol_typedesc (const ShaderSymbol *sym) const;
 
     /// Given a context (that has executed a shader) and an opaque
-    /// ShserSymbol*, return the actual memory address where the value of
+    /// ShaderSymbol*, return the actual memory address where the value of
     /// the symbol resides within the heap memory of the context. This
     /// is only valid for the shader execution that had happened immediately
     /// prior for this context, but it is a very inexpensive operation.
@@ -552,6 +648,13 @@ public:
     bool query_closure (const char **name, int *id,
                         const ClosureParam **params);
 
+    /// For the proposed shader "global" name, return the corresponding
+    /// SGBits enum.
+    static SGBits globals_bit (ustring name);
+
+    /// For the SGBits value, return the shader "globals" name.
+    static ustring globals_name (SGBits bit);
+
     /// For the proposed raytype name, return the bit pattern that
     /// describes it, or 0 for an unrecognized name.  (This retrieves
     /// data passed in via attribute("raytypes")).
@@ -564,19 +667,20 @@ public:
     /// to the optimizer, and will be determined strictly at execution time.
     void set_raytypes(ShaderGroup *group, int raytypes_on, int raytypes_off);
 
-    /// Ensure that the group has been optimized and JITed.
-    /// Ensure that the group has been optimized and JITed.
-    void optimize_group (ShaderGroup *group);
+    /// Ensure that the group has been optimized and optionally JITed. The ctx pointer
+    /// supplies a ShadingContext to use.
+    void optimize_group (ShaderGroup *group, ShadingContext *ctx, bool do_jit = true);
 
-    /// Ensure that the group has been optimized and JITed. This is a
-    /// convenience function that simply calls set_raytypes followed by optimize_group.
+    /// Ensure that the group has been optimized and optionally JITed. This is a
+    /// convenience function that simply calls set_raytypes followed by
+    /// optimize_group. The ctx supplies a ShadingContext to use.
     void optimize_group (ShaderGroup *group, int raytypes_on,
-                         int raytypes_off);
+                         int raytypes_off, ShadingContext *ctx, bool do_jit = true);
 
     /// If option "greedyjit" was set, this call will trigger all
     /// shader groups that have not yet been compiled to do so with the
     /// specified number of threads (0 means use all available HW cores).
-    void optimize_all_groups (int nthreads=0);
+    void optimize_all_groups (int nthreads=0, bool do_jit = true);
 
     /// Return a pointer to the TextureSystem being used.
     TextureSystem * texturesys () const;
@@ -586,6 +690,9 @@ public:
 
     /// Archive the entire shader group so that it can be reconstituted
     /// later.
+    bool archive_shadergroup (ShaderGroup &group, string_view filename);
+
+    // DEPRECATED(2.0)
     bool archive_shadergroup (ShaderGroup *group, string_view filename);
 
     /// Helper function -- copy or convert a source value (described by
@@ -602,33 +709,36 @@ public:
     /// assignment in OSL itself:
     ///   int -> float             convert to float
     ///   int -> triple            convert to float and replicate x3
+    ///   int -> float[4]          convert to float and replicate x4
     ///   float -> triple          replicate x3
+    ///   float -> float[4]        replicate x4
     ///   float -> int             truncate like a (int) type cast
-    ///   triple -> triple         copy, regarless of differing vector types
+    ///   triple -> triple         copy, regardless of differing vector types
     /// 3. Additional rules not allowed in OSL source code:
     ///   float -> float[2]        replicate x2
     ///   int -> float[2]          convert to float and replicate x2
     ///   float[2] -> triple       (f[0], f[1], 0)
+    ///   float[4] -> vec4         allow conversion to OIIO type (no vec4 in OSL)
     ///
     /// Observation: none of the supported conversions require more
     /// storage for src than for dst.
-    static bool convert_value (void *dst, TypeDesc dsttype,
-                               const void *src, TypeDesc srctype);
+    static bool convert_value(void* dst, TypeDesc dsttype, const void* src,
+                              TypeDesc srctype);
 
 private:
-    pvt::ShadingSystemImpl *m_impl;
+    pvt::ShadingSystemImpl* m_impl;
 };
 
 
 
-#ifdef OPENIMAGEIO_IMAGEBUF_H
+#ifdef OPENIMAGEIO_IMAGEBUFALGO_H
 // To keep from polluting all OSL clients with ImageBuf & ROI, only expose
-// the following declarations if they have included OpenImageIO/imagebuf.h.
+// the following declarations if they have included OpenImageIO/imagebufalgo.h.
 
 // enum describing where shades are located for shade_image().
 enum ShadeImageLocations {
-    ShadePixelCenters,   // locate shades at pixel centers: (i+0.5)/res
-    ShadePixelGrid       // locate shades at grid nodes: i/(res-1)
+    ShadePixelCenters,  // locate shades at pixel centers: (i+0.5)/res
+    ShadePixelGrid      // locate shades at grid nodes: i/(res-1)
 };
 
 
@@ -652,11 +762,13 @@ enum ShadeImageLocations {
 /// as if it were a grid that is shaded at exact endpoints (position
 /// i/(res+1)). In either case, derivatives will be set appropriately.
 OSLEXECPUBLIC
-bool shade_image (ShadingSystem &shadingsys, ShaderGroup &group,
-                  const ShaderGlobals *defaultsg,
-                  OIIO::ImageBuf &buf, OIIO::array_view<ustring> outputs,
-                  ShadeImageLocations shadelocations = ShadePixelCenters,
-                  OIIO::ROI roi = OIIO::ROI(), int nthreads = 0);
+bool
+shade_image(ShadingSystem& shadingsys, ShaderGroup& group,
+            const ShaderGlobals* defaultsg, OIIO::ImageBuf& buf,
+            cspan<ustring> outputs,
+            ShadeImageLocations shadelocations              = ShadePixelCenters,
+            OIIO::ROI roi                                   = OIIO::ROI(),
+            OIIO::ImageBufAlgo::parallel_image_options popt = 0);
 
 #endif
 
